/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.print;

import java.rmi.RemoteException;

import org.eclipse.nebula.paperclips.core.EmptyPrint;
import org.eclipse.nebula.paperclips.core.LayerPrint;
import org.eclipse.nebula.paperclips.core.LinePrint;
import org.eclipse.nebula.paperclips.core.Print;
import org.eclipse.nebula.paperclips.core.border.BorderPrint;
import org.eclipse.nebula.paperclips.core.border.LineBorder;
import org.eclipse.nebula.paperclips.core.grid.DefaultGridLook;
import org.eclipse.nebula.paperclips.core.grid.GridPrint;
import org.eclipse.nebula.paperclips.core.page.PagePrint;
import org.eclipse.nebula.paperclips.core.text.TextPrint;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;

import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelSizeEnum;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.frankmuenster.jameica.orverw.util.Utils;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Mnster
 */
public class PrintLabel extends AbstractPrint {

    /**
     * Der Ordner dessen Label gedruckt werden soll
     */
    private final Ordner ordner; // NOPMD

    /**
     * Der Titel
     */
    private final String titel; // NOPMD

    /**
     * Die Gre des Labels
     */
    private final LabelSizeEnum size; // NOPMD

    /**
     * die Hintergrundfarbe
     */
    private final RGB lblBgColor; // NOPMD

    /**
     * Der Konstruktor
     *
     * @param ordner
     *            der Ordner dessen Label gedruckt werden soll
     * @throws ApplicationException
     */
    public PrintLabel(final Ordner ordner) throws ApplicationException {

        // Call the super class
        super();
        // store the
        this.ordner = ordner;
        try {
            this.size = LabelSizeEnum.getLabelSize(ordner.getWidth());
            this.lblBgColor = ordner.getLabelBackground().getRGB();
            this.titel = ordner.getTitel();
        } catch (final RemoteException e) {
            throw new ApplicationException(I18N.tr("PrintLabel.errorCreatingPrintObj"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see AbstractPrint#getTitle()
     */
    @Override
    public String getTitle() {

        return Utils.format(this.titel, 10);
    }

    /**
     * @see AbstractPrint#customise(PagePrint)
     */
    @Override
    protected void customise(final PagePrint page) throws ApplicationException {

        // hier brauchen wir nix tun tun
    }

    /**
     * @see AbstractPrint#getPrintContent()
     */
    @Override
    protected Print getPrintContent() throws ApplicationException {

        final DefaultGridLook look = new DefaultGridLook();
        look.setBodyBackground(this.lblBgColor);
        look.setCellPadding(4, 0);
        final int width = (int) ((this.size.getWidth() - 8) * 0.039 * 72);
        final int topMargin = (int) (15 * 0.039 * 72);
        final int titleMargin = (int) (20 * 0.039 * 72);
        final int contentMargin = (int) (80 * 0.039 * 72);
        final int idHeight = (int) (8 * 0.039 * 72);
        final GridPrint outer = new GridPrint(String.valueOf(width + 10));
        final GridPrint grid = new GridPrint(String.valueOf(width), look);
        try {
            final EmptyPrint topP = new EmptyPrint(width, topMargin);

            // Bar1
            final LinePrint barP1 = new LinePrint(SWT.HORIZONTAL, (float) 4.0);
            if (this.ordner.isBarVisible()) {
                barP1.setRGB(this.ordner.getBarColor().getRGB());
            } else {
                barP1.setRGB(this.lblBgColor);
            }

            // Title
            final GridPrint child = new GridPrint(width + ", 0mm", look); //$NON-NLS-1$
            final EmptyPrint titleEmpty = new EmptyPrint(0, titleMargin);
            final TextPrint titleTextPrint = new TextPrint(this.ordner.getTitel(),
                    this.ordner.getTitelFont().getFontData()[0], this.ordner.getTitelAlignment());
            titleTextPrint.setForeground(this.ordner.getTitelColor().getRGB());
            child.add(this.ordner.getTitelAlignment(), SWT.CENTER, titleTextPrint);
            child.add(titleEmpty);

            // Bar2
            final LinePrint barP2 = new LinePrint(SWT.HORIZONTAL, (float) 4.0);
            if (this.ordner.isBarVisible()) {
                barP2.setRGB(this.ordner.getBarColor().getRGB());
            } else {
                barP2.setRGB(this.lblBgColor);
            }
            // Content
            final EmptyPrint contentEmpty = new EmptyPrint(width, contentMargin);
            final TextPrint contentTextPrint = new TextPrint("\n" + this.ordner.getContent(), //$NON-NLS-1$
                    this.ordner.getContentFont().getFontData()[0], this.ordner.getContentAlignment());
            contentTextPrint.setForeground(this.ordner.getContentColor().getRGB());
            final LayerPrint contentLayerPrint = new LayerPrint();
            contentLayerPrint.add(contentEmpty);
            contentLayerPrint.add(contentTextPrint, this.ordner.getContentAlignment());

            // Footer
            final EmptyPrint footerEmpty = new EmptyPrint(width, topMargin);
            final TextPrint footerTextPrint = new TextPrint(this.ordner.getFooter(),
                    this.ordner.getFooterFont().getFontData()[0], this.ordner.getFooterAlignment());
            footerTextPrint.setForeground(this.ordner.getFooterColor().getRGB());
            final LayerPrint footerLayerPrint = new LayerPrint();
            footerLayerPrint.add(footerEmpty);
            footerLayerPrint.add(footerTextPrint, this.ordner.getFooterAlignment());

            // ID
            final GridPrint idChild = new GridPrint(width + ", 0mm", look); //$NON-NLS-1$
            final EmptyPrint idEmpty = new EmptyPrint(0, idHeight);
            final TextPrint idTextPrint = new TextPrint("ID " + this.ordner.getID(), //$NON-NLS-1$
                    new FontData("Arial", 10, SWT.NORMAL), SWT.RIGHT); //$NON-NLS-1$
            if (this.ordner.isIdVisible()) {
                idChild.add(SWT.RIGHT, SWT.TOP, idTextPrint);
            }
            idChild.add(idEmpty);

            // add all pieces to the inner grid
            grid.add(topP);
            grid.add(barP1);
            grid.add(child);
            grid.add(barP2);
            grid.add(contentLayerPrint);
            grid.add(footerLayerPrint);
            grid.add(idChild);

            final LineBorder lineBorder = new LineBorder();
            lineBorder.setGapSize(1);
            lineBorder.setLineWidth(1);
            final BorderPrint gridBorderPrint = new BorderPrint(grid, lineBorder);
            outer.add(gridBorderPrint);
        } catch (

        final RemoteException e) {
            throw new ApplicationException(I18N.tr("PrintLabel.errorCreatingPrintContent", this.titel), e); //$NON-NLS-1$
        }
        return outer;
    }
}
